# -*- coding: utf-8 -*-
"""
Copy database.sqlite in a new database named database_saved.sqlite.

USAGE
    python -m thyrsis.database.convert_database database.sqlite

    copy database.sqlite in a new database named database_saved.sqlite.
    if database_saved.sqlite already exists, it will be replaced.
    Then modify database.sqlite.
"""

import os
import shutil
from ..database import sqlite
from ..database import create_site_tables
from ..log import logger

def columns(cursor, table, exclude=[]):
    """ Return columns name for a table
    
    :param cursor: cursor on a sqlite connection
    :type cursor: sqlite3.Cursor
    :param table: name of the table
    :type table: string
    :param exclude: list of excluded columns
    :type exclude: list

    :return: list of the table columns.
    :rtype: list
    """
    return [c for [dum0, c, dum1, dum2, dum3, dum4]
            in cursor.execute("pragma table_info('%s')"%(table)).fetchall() 
            if not c in exclude]

def convert_database(sitesdb):
    """ Convert a database
    
    :param sitesdb: path of the databse to convert
    :type sitesdb: string

    :return: None
    """
    #convert site database
    sitesdb_saved = sitesdb[:-7]+"_saved.sqlite"

    # fix_print_with_import
    print("copying spatialite database ", sitesdb, " into ", sitesdb_saved)
    shutil.copyfile(sitesdb, sitesdb_saved)

    with sqlite.connect(sitesdb, "") as src:
        src_cur = src.cursor()

        names = [i[0] for i in src_cur.execute("""select name from pragma_table_info("metis_options")""").fetchall()]
        if "critereres" not in names:
            # ajout de l'option critereres
            # fix_print_with_import
            print("copying table metis_options adding critereres")
            src_cur.execute("ALTER TABLE metis_options RENAME TO metis_options_orig")
            src_cur.execute("""
                CREATE TABLE metis_options(
                    id INTEGER PRIMARY KEY REFERENCES simulations(id) ON UPDATE CASCADE ON DELETE CASCADE,
                    niter_gc_zns INTEGER DEFAULT 100,
                    niter_gc_zs INTEGER DEFAULT 100,
                    saturation_initiale REAL DEFAULT 0.8,
                    rmult_cormax REAL DEFAULT 0.1,
                    c_pstep REAL DEFAULT 0.1,
                    c_ps_inc REAL DEFAULT 0.1,
                    c_ps_tol REAL DEFAULT 1.e-5,
                    ratio_max_res REAL DEFAULT 1.e-6,
                    critereres REAL DEFAULT 1.e-5,
                    niterns INTEGER DEFAULT 1000,
                    courant TEXT DEFAULT 'non' CHECK (courant IN ('oui', 'non')),
                    emmagasinement REAL DEFAULT 1.e-3,
                    kdecoup_delt_zns REAL DEFAULT 10.,
                    kdecoup_dmin_zns REAL DEFAULT 1.,
                    kdecoup_delt_zs REAL DEFAULT 100.,
                    kdecoup_dmin_zs REAL DEFAULT 1.,
                    force_fin_phase TEXT DEFAULT 'oui' CHECK (force_fin_phase IN ('oui', 'non')),
                    max_iter INTEGER DEFAULT 200,
                    relax_phreat REAL DEFAULT 0.5,
                    critere_pot REAL DEFAULT 0.1,
                    phrea_min REAL DEFAULT 1.0)
            """)

            src_cur.execute("INSERT INTO metis_options(%(cols)s) SELECT %(cols)s \
                    FROM metis_options_orig"%{
                    'cols' : ','.join(columns(src_cur, 'metis_options', exclude=['critereres']))})
            src_cur.execute("UPDATE metis_options SET critereres = 1e-5")
            src_cur.execute("DROP TABLE metis_options_orig")

            #src_cur.execute("INSERT INTO main.parametres(%(cols)s) SELECT %(cols)s \
            #        FROM src.parametres"%{
            #        'cols' : ','.join(columns(src_cur, 'parametres'))})

        if False :
            # ajout du paramètre VGE (air-entry in Ippisch-Vogel formulation)
            # fix_print_with_import
            print("modifying table parametres adding VGE")
            tup = src_cur.execute("SELECT * FROM parametres").fetchall()
            values = []
            idx = 1
            for t in tup:
                values.append([idx] + list(t[1:]))
                idx += 1
                if 'VGS' in t:
                    values.append([idx, t[1], u'VGE', 0, 0, 0, u'constant'])
                    idx += 1

            src_cur.execute("DELETE FROM parametres")
            src_cur.executemany("INSERT INTO parametres(%(cols)s) VALUES (?,?,?,?,?,?,?)"%{
                    'cols' : ','.join(columns(src_cur, 'parametres'))}, values)

            if src_cur.execute("PRAGMA table_info('parametres_simulation')").fetchone():
                # fix_print_with_import
                print("modifying table parametres_simulation adding VGE")
                src_cur.execute("ALTER TABLE parametres_simulation RENAME TO parametres_simulation_orig")
                src_cur.execute("""
                    CREATE TABLE parametres_simulation(
                        id INTEGER PRIMARY KEY,
                        wc REAL NOT NULL,
                        vga REAL NOT NULL,
                        vgn REAL NOT NULL,
                        vgr REAL NOT NULL,
                        vgs REAL NOT NULL,
                        vge REAL DEFAULT 0,
                        wt REAL NOT NULL,
                        vm REAL NOT NULL,
                        dk REAL NOT NULL,
                        sl REAL NOT NULL,
                        dlzns REAL NOT NULL,
                        dlzs REAL NOT NULL,
                        wm REAL NOT NULL)""")

                src_cur.execute("INSERT INTO parametres_simulation(%(cols)s) SELECT %(cols)s \
                    FROM parametres_simulation_orig"%{
                    'cols' : ','.join(columns(src_cur, 'parametres_simulation', exclude=['vge']))})
                #cur.execute("UPDATE parametres_simulation SET vge = 0")
                src_cur.execute("DROP TABLE parametres_simulation_orig")

        src.commit()

        tmp_db = sitesdb[:-7]+"_tmp.sqlite"
        if os.path.exists(tmp_db):
            os.remove(tmp_db)
        with sqlite.connect(tmp_db, "") as dst:
            dst_cur = dst.cursor()
            dst_cur.execute("SELECT InitSpatialMetaData(1)")
            create_site_tables(dst_cur, with_cid=True)
            dst.commit()

            tables=['dates_resultats','elements_chimiques','injections_second_milieu','mesures',
                'metis_options', 'openfoam_options', 'parametres', 'simulations',
                'simulations_second_milieu', 'sites']
            for t in tables:
                rows = src_cur.execute("SELECT * FROM {}".format(t)).fetchall()
                fields = [i[0] for i in src_cur.execute("""SELECT name FROM pragma_table_info('{}')""".format(t)).fetchall()]
                fields_str = ','.join(fields)
                dst_cur.executemany("INSERT INTO {}({}) VALUES ({})".format(t,fields_str,','.join(['?' for i in fields])), rows)
            dst.commit()
            gtables= ['forages', 'fracturations','injections','lithologies','points_interet','stratigraphies']
            for g in gtables:
                gfields = [i[0].upper() for i in src_cur.execute("SELECT f_geometry_column FROM geometry_columns WHERE f_table_name LIKE '{}'".format(g)).fetchall()]
                fields = [i[0] for i in src_cur.execute("""SELECT name FROM pragma_table_info('{}')""".format(g)).fetchall() if i[0] not in gfields]
                fields_str = ','.join(fields)
                gfields_str = ','.join(gfields)
                trans_gfields_str = ','.join(['ST_TRANSFORM({},4326) as {}'.format(i,i) for i in gfields])
                rows = src_cur.execute("SELECT {},{} FROM {}".format(fields_str,trans_gfields_str,g)).fetchall()
                dst_cur.executemany("INSERT INTO {}({},{}) VALUES ({})".format(g,fields_str,gfields_str,','.join(['?' for i in fields+gfields])), rows)
            dst.commit()

    os.remove(sitesdb)
    os.rename(tmp_db,sitesdb)

# run as script if invoqued as such
if __name__ == "__main__":
    import sys
    import sip
    assert(sip.getapi('QVariant') == 2)

    if len(sys.argv) == 2 and sys.argv[1] in ["-h","--help"]:
        help(sys.modules[__name__])
        exit(0)

    if len(sys.argv) not in [2]:
        logger.error("wrong number of arguments (try compute.py --help)")
        exit(1)

    convert_database(sys.argv[1])
